@@ -68,7 +68,8 @@ char *pfield(char *hdr_line, int pos)
 			j = strcspn(hl, "&");
 			*(hl + j) = ';';
 		}
-		strcpy(field, hl);
+		strncpy(field, hl, HEADER_LINE_LEN);
+		field[HEADER_LINE_LEN - 1] = '\0';
 	}
 
 	/* Display current field */
